"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createController = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
function createController(gcpFunctions) {
    let FunctionController = class FunctionController {
        constructor() {
            this.logger = new common_1.Logger('Runner');
            this.endpoints = gcpFunctions;
            for (const { endpoint, trigger } of this.endpoints) {
                this.logger.debug(`Register ${trigger} -> "${endpoint}"`);
            }
        }
        get(req, res) {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.logger.log(`[GET] Handle "${req.path}"`);
                return this.executeFunction(req, res);
            });
        }
        post(req, res) {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                this.logger.log(`[POST] Handle "${req.path}"`);
                return this.executeFunction(req, res);
            });
        }
        option(req, res) {
            this.logger.log(`[OPTIONS] Handle "${req.path}"`);
            return this.executeFunction(req, res);
        }
        delete(req, res) {
            this.logger.log(`[DELETE] Handle "${req.path}"`);
            return this.executeFunction(req, res);
        }
        executeFunction(req, res) {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                const endpoint = this.endpoints.find(({ endpoint }) => (req.path.startsWith(endpoint)));
                if (endpoint) {
                    if (endpoint.trigger === 'http') {
                        return endpoint.func(req, res);
                    }
                    else if (endpoint.trigger === 'topic') {
                        return this.simulatePubSubEvent(req, res, endpoint.func);
                    }
                    this.logger.warn(`"${req.path}" unsupported trigger!`);
                }
                else {
                    this.logger.warn(`"${req.path}" not found!`);
                }
                res
                    .status(404)
                    .send('Function not found!');
            });
        }
        simulatePubSubEvent(req, res, pubSub) {
            return tslib_1.__awaiter(this, void 0, void 0, function* () {
                if (req.method === 'POST'
                    && req.headers['content-type'].includes('application/json')
                    && Object.getPrototypeOf(req.body) === Object.prototype) {
                    try {
                        let message = { attributes: null, data: null, timestamp: new Date() };
                        if (req.body.message) {
                            message = req.body.message;
                        }
                        if (req.body.data) {
                            message.data = Buffer.from(Object.getPrototypeOf(req.body.data) === Object.prototype
                                ? JSON.stringify(req.body.data)
                                : req.body.data, 'binary').toString('base64');
                        }
                        const response = yield pubSub(message);
                        if (response) {
                            res.send(response);
                        }
                        else {
                            res.send('ok');
                        }
                    }
                    catch (err) {
                        console.error(err);
                        res.status(500)
                            .send(err);
                    }
                }
                res.status(405)
                    .send();
            });
        }
    };
    tslib_1.__decorate([
        (0, common_1.Get)('/*'),
        tslib_1.__param(0, (0, common_1.Req)()),
        tslib_1.__param(1, (0, common_1.Res)()),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [Object, Object]),
        tslib_1.__metadata("design:returntype", Promise)
    ], FunctionController.prototype, "get", null);
    tslib_1.__decorate([
        (0, common_1.Post)('/*'),
        tslib_1.__param(0, (0, common_1.Req)()),
        tslib_1.__param(1, (0, common_1.Res)()),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [Object, Object]),
        tslib_1.__metadata("design:returntype", Promise)
    ], FunctionController.prototype, "post", null);
    tslib_1.__decorate([
        (0, common_1.Options)('/*'),
        tslib_1.__param(0, (0, common_1.Req)()),
        tslib_1.__param(1, (0, common_1.Res)()),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [Object, Object]),
        tslib_1.__metadata("design:returntype", Promise)
    ], FunctionController.prototype, "option", null);
    tslib_1.__decorate([
        (0, common_1.Delete)('/*'),
        tslib_1.__param(0, (0, common_1.Req)()),
        tslib_1.__param(1, (0, common_1.Res)()),
        tslib_1.__metadata("design:type", Function),
        tslib_1.__metadata("design:paramtypes", [Object, Object]),
        tslib_1.__metadata("design:returntype", Promise)
    ], FunctionController.prototype, "delete", null);
    FunctionController = tslib_1.__decorate([
        (0, common_1.Controller)(),
        tslib_1.__metadata("design:paramtypes", [])
    ], FunctionController);
    return FunctionController;
}
exports.createController = createController;
//# sourceMappingURL=__runner.controller.js.map